<?php

/**
 * Social networking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Social networking management
 *
 * @package 	PG_RealEstate
 * @subpackage 	Social networking
 *
 * @category	helpers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class OAUTH_ALGORITHMS
{
    const HMAC_SHA1 = 'HMAC-SHA1';
    const RSA_SHA1 = 'RSA-SHA1';
}

if (!function_exists('sign_hmac_sha1')) {
    /**
     * Encode method by hmac
     *
     * @param string $method     method name
     * @param string $baseurl    base url
     * @param string $secret     application secret
     * @param array  $parameters encode options
     *
     * @return string
     */
    function sign_hmac_sha1($method, $baseurl, $secret, array $parameters)
    {
        $data = $method . '&';
        $data .= urlencode($baseurl) . '&';
        $oauth = '';
        ksort($parameters);
        if (!array_key_exists('oauth_token_secret', $parameters)) {
            $parameters['oauth_token_secret'] = '';
        }
        foreach ($parameters as $key => $value) {
            if (strtolower($key) != 'oauth_token_secret') {
                $oauth .= "&{$key}={$value}";
            }
        }
        $data .= urlencode(substr($oauth, 1));
        $secret .= '&' . $parameters['oauth_token_secret'];

        return base64_encode(hash_hmac('sha1', $data, $secret, true));
    }
}

if (!function_exists('sign_rsa_sha1')) {
    /**
     * Encode method by rsa
     *
     * @param string $method     method name
     * @param string $baseurl    base url
     * @param string $certfile   certificate path
     * @param array  $parameters encode options
     *
     * @return string
     */
    function sign_rsa_sha1($method, $baseurl, $certfile, array $parameters)
    {
        $fp = fopen($certfile, "r");
        $private = fread($fp, 8192);
        fclose($fp);
        $data = $method . '&';
        $data .= urlencode($baseurl) . '&';
        $oauth = '';
        ksort($parameters);
        foreach ($parameters as $key => $value) {
            $oauth .= "&{$key}={$value}";
        }
        $data .= urlencode(substr($oauth, 1));
        $keyid = openssl_get_privatekey($private);
        openssl_sign($data, $signature, $keyid);
        openssl_free_key($keyid);

        return base64_encode($signature);
    }
}

if (!function_exists('build_auth_string')) {
    /**
     * Build oauth authorization header
     *
     * @param array $authparams authorize parameters
     *
     * @return string
     */
    function build_auth_string(array $authparams)
    {
        $header = "Authorization: OAuth ";
        $auth = '';
        foreach ($authparams as $key => $value) {
            if ($key != 'oauth_token_secret') {
                $auth .= ", {$key}=\"{$value}\"";
            }
        }

        return $header . substr($auth, 2) . "\r\n";
    }
}

if (!function_exists('build_auth_array')) {
    /**
     * Build oauth authorization post parameters
     *
     * @param string $baseurl base url
     * @param string $key     application key
     * @param string $secret  application secret
     * @param array  $extra   extra parameters
     * @param string $method  method name
     * @param string $algo    encode algorithm
     *
     * @return string
     */
    function build_auth_array($baseurl, $key, $secret, $extra = array(), $method = 'GET', $algo = OAUTH_ALGORITHMS::RSA_SHA1)
    {
        $auth['oauth_consumer_key'] = $key;
        $auth['oauth_signature_method'] = $algo;
        $auth['oauth_timestamp'] = time();
        $auth['oauth_nonce'] = md5(uniqid(rand(), true));
        $auth['oauth_version'] = '1.0';
        $auth = array_merge($auth, $extra);
        $urlsegs = explode("?", $baseurl);
        $baseurl = $urlsegs[0];
        $signing = $auth;
        if (count($urlsegs) > 1) {
            preg_match_all("/([\w\-]+)\=([\w\d\-\%\.\$\+\*]+)\&?/", $urlsegs[1], $matches);
            $signing = $signing + array_combine($matches[1], $matches[2]);
        }
        if (strtoupper($algo) == OAUTH_ALGORITHMS::HMAC_SHA1) {
            $auth['oauth_signature'] = sign_hmac_sha1($method, $baseurl, $secret, $signing);
        } elseif (strtoupper($algo) == OAUTH_ALGORITHMS::RSA_SHA1) {
            $auth['oauth_signature'] = sign_rsa_sha1($method, $baseurl, $secret, $signing);
        }
        $auth['oauth_signature'] = urlencode($auth['oauth_signature']);

        return $auth;
    }
}

if (!function_exists('get_auth_header')) {
    /**
     * Build oauth authorization header (helper)
     *
     * @param string $baseurl base url
     * @param string $key     application key
     * @param string $secret  application secret
     * @param array  $extra   extra parameters
     * @param string $method  method name
     * @param string $algo    encode algorithm
     *
     * @return string
     */
    function get_auth_header($baseurl, $key, $secret, $extra = array(), $method = 'GET', $algo = OAUTH_ALGORITHMS::RSA_SHA1)
    {
        $auth = build_auth_array($baseurl, $key, $secret, $extra, $method, $algo);

        return build_auth_string($auth);
    }
}

if (!function_exists('show_social_networks_head')) {
    /**
     * Display social networks header
     *
     * @return void
     */
    function show_social_networks_head()
    {
        $ci = &get_instance();
        $ci->load->model('social_networking/models/Social_networking_widgets_model');
        return $ci->Social_networking_widgets_model->get_header();
    }
}

if (!function_exists('show_social_networks_like')) {
    /**
     * Display social networks 'like' widgets
     *
     * @param boolean $disable_header no header
     *
     * @return void
     */
    function show_social_networks_like($disable_header = false)
    {
        $ci = &get_instance();
        $ci->load->model('social_networking/models/Social_networking_pages_model');
        $page_data = $ci->Social_networking_pages_model->get_page_data($ci->router->class, $ci->router->method);
        if ($page_data) {
            $ci->load->model('social_networking/models/Social_networking_widgets_model');
            $like = $ci->Social_networking_widgets_model->get_widgets('like', isset($page_data['data']['like']) ? $page_data['data']['like'] : array());
            $ci->view->assign('like', $like);
            $ci->view->assign('disable_header', $disable_header);
            $ci->view->render('like', 'user', 'social_networking');
        }
    }
}

if (!function_exists('show_social_networks_share')) {
    /**
     * Display social networks 'share' widgets
     *
     *
     * @return void
     */
    function show_social_networks_share()
    {
        $ci = &get_instance();
        $ci->load->model('social_networking/models/Social_networking_pages_model');
        $page_data = $ci->Social_networking_pages_model->get_page_data($ci->router->class, $ci->router->method);
        if ($page_data) {
            $ci->load->model('social_networking/models/Social_networking_widgets_model');
            $share = $ci->Social_networking_widgets_model->get_widgets('share', isset($page_data['data']['share']) ? $page_data['data']['share'] : array());
            $ci->view->assign('share', $share);
            $ci->view->render('share', 'user', 'social_networking');
        }
    }
}

if (!function_exists('show_social_networks_comments')) {
    /**
     * Display social networks 'comments' widgets
     *
     * @return void
     */
    function show_social_networks_comments()
    {
        $ci = &get_instance();
        $ci->load->model('social_networking/models/Social_networking_pages_model');
        $page_data = $ci->Social_networking_pages_model->get_page_data($ci->router->class, $ci->router->method);
        if ($page_data) {
            $ci->load->model('social_networking/models/Social_networking_widgets_model');
            $comments = $ci->Social_networking_widgets_model->get_widgets('comments', isset($page_data['data']['comments']) ? $page_data['data']['comments'] : array(), 'column');
            $ci->view->assign('comments', $comments);
            $ci->view->render('comments', 'user', 'social_networking');
        }
    }
}
