<?php

namespace Pg\Modules\Users\Controllers;

use Pg\Modules\Users\Acl;
/**
 * Users module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Users user side controller
 *
 * @package     PG_RealEstate
 * @subpackage  Users
 *
 * @category    controllers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users extends \Controller
{
    const REDIRECT_KEY = 'redirect_after_login';

    /**
     * Constructor
     *
     * @return Users
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Users_model");
        $this->load->model("Menu_model");
    }

    /**
     * Sign in form by ajax
     *
     * @return void
     */
    public function ajax_login_form()
    {
        $this->load->helper('users');
        if (!is_login_enabled()) {
            return "";
        }

        $user_types_groups = $this->Users_model->getUserTypesGroups();

        $user_types = [];
        foreach ($this->Users_model->getUserTypes() as $user_type) {
            if ($this->pg_module->get_module_config("users", $user_type . "_register_enabled")) {
                $user_types[] = $user_type;
            }
        }

        $redirect = filter_input(INPUT_POST, self::REDIRECT_KEY, FILTER_SANITIZE_URL);
        if (!empty($redirect)) {
            $this->session->set_userdata(self::REDIRECT_KEY, $redirect);
        }
        $this->view->assign("user_types_groups", $user_types_groups);
        $this->view->assign("user_types", $user_types);
        $this->view->render("ajax_login_form");
    }

    /**
     * Sign in form
     *
     * @return void
     */
    public function login_form()
    {
        $this->load->helper('users');
        if (!is_login_enabled()) {
            show_404();
        }

        if ($this->session->userdata('auth_type') == 'user') {
            $this->view->setRedirect(site_url() . 'start/homepage');
            return;
        }

        $user_types = array();
        foreach ($this->Users_model->getUserTypes() as $user_type) {
            if ($this->pg_module->get_module_config("users", $user_type . "_register_enabled")) {
                $user_types[] = $user_type;
            }
        }
        $this->view->assign("user_types", $user_types);

        $this->Menu_model->breadcrumbs_set_active(l("header_login", "users"));
        $this->view->render("login_form");
    }

    private function getRedirectAfterLoginUrlByCurrentSession()
    {
        $redirect_url = filter_var($this->session->userdata(self::REDIRECT_KEY), FILTER_SANITIZE_URL);
        if (!$redirect_url || strpos($redirect_url, site_url()) !== 0) {
            if (strpos($_SERVER["HTTP_REFERER"], site_url()) !== false && !strstr(str_replace(site_url(), '',
                                                                                              $_SERVER["HTTP_REFERER"]),
                                                                                              'start/error')) {
                $redirect_url = $_SERVER["HTTP_REFERER"];
            } else {
                $redirect_url = site_url() . "start/homepage";
            }
        } else {
            $this->session->set_userdata(self::REDIRECT_KEY, null);
        }
        return $redirect_url;
    }

    /**
     * Sign in to account
     *
     * @return void
     */
    public function login()
    {
        $errors = array();
        $this->load->model('users/models/Auth_model');

        $data = array(
            'email' => trim(strip_tags($this->input->post('email', true))),
            'password' => trim(strip_tags($this->input->post('password', true))),
        );

        $redirect_url = $this->getRedirectAfterLoginUrlByCurrentSession();

        $validate = $this->Auth_model->validate_login_data($data);
        if (!empty($validate['errors'])) {
            $errors = $validate['errors'];
        } else {
            $login_return = $this->Auth_model->login_by_email_password(
                    $validate['data']['email'], $this->Users_model->encode_password($validate['data']['password'])
            );
            if (!empty($login_return['errors'])) {
                $errors = $login_return['errors'];
            }
        }
        if (!empty($errors)) {
            foreach ($errors as $error) {
                $this->system_messages->addMessage(View::MSG_ERROR, $error);
            }
            $redirect_url = site_url();
        }
        $this->view->setRedirect($redirect_url);
    }

    /**
     * Log out from account
     *
     * @return void
     */
    public function logout()
    {
        $this->load->model('users/models/Auth_model');
        $lang_id = $this->session->userdata('lang_id');
        $this->Auth_model->logoff();
        $this->session->sess_create();
        if ($this->session->userdata('lang_id') != $lang_id) {
            $this->session->set_userdata('lang_id', $lang_id);
        }
        $this->view->setRedirect();
    }

    /**
     * Change user language
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function change_language($lang_id)
    {
        $lang_id = intval($lang_id);

        $old_code = $this->pg_language->languages[$this->pg_language->current_lang_id]["code"];
        $this->pg_language->current_lang_id = $lang_id;
        $code = $this->pg_language->languages[$lang_id]["code"];
        $_SERVER["HTTP_REFERER"] = str_replace("/$old_code/", "/$code/", $_SERVER["HTTP_REFERER"]);

        $this->session->set_userdata("lang_id", $lang_id);

        if ($this->session->userdata("auth_type") === "user") {
            $user_id = $this->session->userdata("user_id");
            $save_data["lang_id"] = $lang_id;
            $this->Users_model->save_user($user_id, $save_data);
        }

        if (strpos($_SERVER["HTTP_REFERER"], site_url()) !== false) {
            $this->view->setRedirect($_SERVER["HTTP_REFERER"]);
        } else {
            $this->view->setRedirect();
        }
    }

    /**
     * Change user language by ajax
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function ajax_change_language($lang_id)
    {
        $lang_id = intval($lang_id);

        $old_code = $this->pg_language->languages[$this->pg_language->current_lang_id]["code"];
        $this->pg_language->current_lang_id = $lang_id;
        $code = $this->pg_language->languages[$lang_id]["code"];
        $_SERVER["HTTP_REFERER"] = str_replace("/$old_code/", "/$code/", $_SERVER["HTTP_REFERER"]);

        $this->session->set_userdata("lang_id", $lang_id);

        if ($this->pg_module->get_module_config('language_identification', 'automatic_identification')) {
            $this->load->helper('cookie');
            set_cookie("lang_id", $lang_id, '0');
        }

        if ($this->session->userdata("auth_type") == "user") {
            $user_id = $this->session->userdata("user_id");
            $save_data["lang_id"] = $lang_id;
            $this->Users_model->save_user($user_id, $save_data);
        }

        $this->view->assign('code_old', $old_code);
        $this->view->assign('code_new', $code);

        $this->view->render();
    }

    /**
     * My account
     *
     * @return void
     */
    public function account()
    {
        $user_id = $this->session->userdata("user_id");

        $this->load->model("users/models/Auth_model");
        $this->Auth_model->update_user_session_data($user_id);

        if ($this->pg_module->is_module_active("payments")) {
            $this->load->model('menu/models/Indicators_model');
            $this->Indicators_model->delete('new_account_item', $user_id, true);
            $this->Indicators_model->delete('all_account_item', 'account_' . $user_id, true);
        }

        $data = $this->Users_model->get_user_by_id($user_id);
        $this->pg_seo->set_seo_data($data);

        // breadcrumbs
        $user_type = $this->session->userdata("user_type");
        $this->Menu_model->breadcrumbs_set_parent($user_type . "_my_account_balance_item");

        $this->view->assign("user_id", $user_id);
        $this->view->assign("data", $data);
        $this->view->render("account");
    }

    /**
     * Services from my account
     *
     * @return void
     */
    public function account_services()
    {
        $user_type = $this->session->userdata('user_type');
        $this->Menu_model->breadcrumbs_set_parent($user_type . '_my_profile_services_item');

        $this->load->model('Services_model');
        $services = $this->Services_model->get_user_type_active_services($user_type);

        $url = site_url() . 'users/account_services';
        $this->session->set_userdata('service_redirect', $url);

        $user_id = $this->session->userdata('user_id');
        $user = $this->Users_model->get_user_by_id($user_id);

        $this->view->assign("is_user_featured", $user['is_featured']);
        $this->view->assign("featured_end_date", $user['featured_end_date']);

        $this->view->assign("is_user_show_logo", $user['is_show_logo']);
        $this->view->assign("show_logo_end_date", $user['show_logo_end_date']);

        $this->load->model('Currencies_model');
        $this->view->assign('base_currency', $this->Currencies_model->get_currency_default(true));
        $this->view->assign("services", $services);
        $this->view->render("account_services");
    }

    /**
     * My profile
     *
     * @param string $action action name
     *
     * @return void
     */
    public function profile($action = "view")
    {
        $user_id = $this->session->userdata("user_id");

        $this->Users_model->set_format_settings(array('get_safe' => false, 'get_company' => true, 'get_field_editor' => true));
        $data = $this->Users_model->get_user_by_id($user_id);
        $this->Users_model->set_format_settings(array('get_safe' => true, 'get_company' => false, 'get_field_editor' => false));

        $user_type = $data['user_type'];

        if ($this->input->post("btn_register")) {
            switch ($action) {
                case 'personal':
                    $post_data = $this->input->post("data", true);
                    $post_data['user_type'] = $user_type;
                    $validate_data = $this->Users_model->validate_user($user_id, $post_data, 'user_icon');
                    if (!empty($validate_data["errors"])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                    } else {
                        if ($this->input->post("user_icon_delete")) {
                            $this->Users_model->delete_logo($user_id);
                        }
                        $this->Users_model->save_user($user_id, $validate_data["data"], 'user_icon', true);
                        $user = $this->Users_model->get_user_by_id($user_id);
                        $this->session->set_userdata('logo',
                                                     $user['media'][!empty($user_data['user_logo_moderation']) ? 'user_logo_moderation' : 'user_logo']['thumbs']['small']);
                        $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_update_user", "users"));

                        $this->load->model("users/models/Auth_model");
                        $this->Auth_model->update_user_session_data($user_id);
                    }
                    break;
                case 'company':
                    $post_data = $this->input->post("data", true);
                    $post_data['user_type'] = $user_type;
                    $validate_data = $this->Users_model->validate_user($user_id, $post_data, 'user_icon');
                    if (!empty($validate_data["errors"])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                    } else {
                        if (!isset($validate_data['data']["agent_company"])) {
                            $validate_data['data']["agent_company"] = 0;
                        }
                        $save_company_status = $this->Users_model->save_agent_company($user_id,
                                                                                      $validate_data['data']["agent_company"]);
                        if ($save_company_status) {
                            $this->system_messages->addMessage(View::MSG_SUCCESS,
                                                               l("success_update_agent_company", "users"));
                        }
                    }
                    break;
                case 'contact':
                    $post_data = $this->input->post("data", true);
                    $fields_for_select = $this->Users_model->get_fields_for_select('user');
                    foreach ($fields_for_select as $field) {
                        $post_data[$field] = $this->input->post($field, true);
                    }
                    $fields_for_select = $this->Users_model->get_fields_for_select($data['user_type']);
                    foreach ($fields_for_select as $field) {
                        $post_data[$field] = $this->input->post($field, true);
                    }
                    $validate_data = $this->Users_model->validate_user($user_id, $post_data);
                    if (!empty($validate_data["errors"])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                    } else {
                        $this->Users_model->save_user($user_id, $validate_data["data"]);
                        $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_update_user", "users"));
                    }
                    break;
                case 'location':
                    $post_data = $this->input->post("data", true);
                    $post_data['user_type'] = $user_type;
                    $validate_data = $this->Users_model->validate_user($user_id, $post_data, 'user_icon');
                    if (!empty($validate_data["errors"])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                    } else {
                        $this->Users_model->save_user($user_id, $validate_data["data"], 'user_icon', true);
                        $this->load->model('geomap/models/Geomap_model');
                        $this->load->model('geomap/models/Geomap_settings_model');
                        $map_gid = $this->Geomap_model->get_default_driver_gid();
                        $post_settings = $this->input->post('map', true);
                        $post_settings['lat'] = $validate_data['data']['lat'];
                        $post_settings['lon'] = $validate_data['data']['lon'];
                        $validate_data = $this->Geomap_settings_model->validate_settings($post_settings);
                        if (!empty($validate_data["errors"])) {
                            $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                        } else {
                            $this->Geomap_settings_model->save_settings($map_gid, 0, $user_id, 'user_profile',
                                                                        $validate_data["data"]);
                            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_update_user", "users"));
                        }
                    }
                    break;
                case 'subscriptions':
                    if ($this->pg_module->is_module_active("subscriptions")) {
                        // Save user subscribers
                        $user_subscriptions_list = $this->input->post("user_subscriptions_list", true);
                        $this->load->model('subscriptions/models/Subscriptions_users_model');
                        $this->Subscriptions_users_model->save_user_subscriptions($user_id, $user_subscriptions_list);
                    }
                    break;
            }

            $action = "view";
        }

        if (!$action) {
            $action = "view";
        }
        $data["action"] = $action;
        $this->view->assign("action", $action);

        $this->pg_seo->set_seo_data($data);

        if ($user_type == "company" || $user_type == "agent") {
            $markers = array(
                array(
                    "country" => isset($data["country"]) ? $data["country"] : 0,
                    "region" => isset($data["region"]) ? $data["region"] : 0,
                    "city" => isset($data["city"]) ? $data["city"] : 0,
                    "district" => isset($data["district"]) ? $data["district"] : 0,
                    "address" => $data["address"],
                    "postal_code" => $data["postal_code"],
                    "lat" => (float) $data["lat"],
                    "lon" => (float) $data["lon"],
                    "info" => (!empty($data['location']) ? $data['location'] : $data["output_name"] . ', ' . l($data['user_type'], 'users')),
                    "dragging" => ($action == "location") ? true : false,
                ),
            );

            $current_language = $this->pg_language->get_lang_by_id($this->pg_language->current_lang_id);
            $view_settings = array(
                "width" => ($action == "location" ? "660" : "565"),
                "height" => "250",
                "lat" => (float) $data["lat"],
                "lon" => (float) $data["lon"],
                "lang" => $current_language["code"],
                "disable_smart_zoom" => 1,
            );
            if ($action == "location") {
                $view_settings['zoom_listener'] = 'get_user_zoom_data';
                $view_settings['type_listener'] = 'get_user_type_data';
                $view_settings['drag_listener'] = 'get_user_drag_data';

                $this->load->model('geomap/models/Geomap_model');
                $this->load->model('geomap/models/Geomap_settings_model');

                $map_gid = $this->Geomap_model->get_default_driver_gid();
                $map_settings = $this->Geomap_settings_model->get_settings($map_gid, 0, $user_id, 'user_profile');
                $this->view->assign('user_map_settings', $map_settings);
            }
            $this->view->assign("map_settings", $view_settings);
            $this->view->assign("markers", $markers);
        }

        switch ($action) {
            case 'contact':
                $this->load->model('Field_editor_model');
                $this->Field_editor_model->initialize('user');
                $user_type_sections = $this->Field_editor_model->get_section_list();

                if ($data['user_type']) {
                    $this->Field_editor_model->initialize($data['user_type']);
                    $fe_sections = $this->Field_editor_model->get_section_list();
                    $user_type_sections = array_merge($user_type_sections, $fe_sections);
                }

                if (!empty($user_type_sections)) {
                    $sections = array();
                    foreach ($user_type_sections as $section) {
                        $sections[] = $section['gid'];
                    }
                    $this->view->assign('sections_data', array_values($user_type_sections));

                    $params['where_in']['section_gid'] = $sections;
                    $fields_data = $this->Field_editor_model->get_form_fields_list($data, $params);
                    $this->view->assign('fields_data', $fields_data);
                }
                break;
        }

        $this->load->model('menu/models/Indicators_model');
        $this->Indicators_model->delete('agent_status_item', $user_id, true);
        $this->Indicators_model->delete('all_profile_item', 'agent_status_' . $user_id, true);

        // breadcrumbs
        $this->Menu_model->breadcrumbs_set_parent($user_type . "_my_profile_item");

        $block = $this->profile_block($action);

        $weekdays = $this->pg_language->ds->get_reference("start", "week_day_full");
        $this->view->assign("weekdays", $weekdays);

        $dayhours = $this->pg_language->ds->get_reference("start", "dayhour-names");
        $this->view->assign("dayhours", $dayhours);

        $this->view->assign("block", $block);
        $this->view->assign("data", $data);
        $this->view->assign("user_type", $user_type);
        $this->view->assign("user_id", $user_id);
        $this->view->assign("edit_mode", 1);
        $this->view->render("profile");
    }

    /**
     * My profile block
     *
     * @param string $action action name
     *
     * @return block
     */
    public function profile_block($action = "view"){

        $user_id = $this->session->userdata("user_id");

        $this->Users_model->set_format_settings(array('get_safe' => false, 'get_company' => true, 'get_field_editor' => true));
        $data = $this->Users_model->get_user_by_id($user_id);
        $this->Users_model->set_format_settings(array('get_safe' => true, 'get_company' => false, 'get_field_editor' => false));

        $page_data["date_format"] = $this->pg_date->get_format('date_literal', 'st');
        $this->view->assign("page_data", $page_data);

        $use_phone_format = $this->pg_module->get_module_config("start", "use_phone_format");
        if ($use_phone_format) {
            $phone_format = $this->pg_module->get_module_config("start", "phone_format");
            $this->view->assign('phone_format', $phone_format);
        }

        $this->view->assign("data", $data);
        $user_type = $data['user_type'];
        $view = ($action == 'view') ?  '_profile' : '_form';
        $module = ($user_type == 'private') ? 'users_private' : 'users';
        return $this->view->fetch($user_type . $view, 'user', $module);
    }

     /**
     * Get profile block
     *
     * @param string    $action       view type
     *
     * @return void
     */
    public function ajax_profile($action = "view")
    {
        $post_data = $this->input->post('data', true);
        $user_id = $this->session->userdata("user_id");

        $this->Users_model->set_format_settings(array('get_safe' => false, 'get_company' => true, 'get_field_editor' => true));
        $data = $this->Users_model->get_user_by_id($user_id);
        $this->Users_model->set_format_settings(array('get_safe' => true, 'get_company' => false, 'get_field_editor' => false));
        $result = array('success' => false);

        foreach ($post_data as $key => $value)
            if ($value == '') unset($post_data[$key]);

        if (!empty($post_data)){
            $validate_data = $this->Users_model->validate_user($user_id, $post_data);
        }
        if (!empty($validate_data["errors"])) {
            $result['message'] = array('text' => $validate_data["errors"], 'type' => View::MSG_ERROR);
        } elseif (isset($post_data["email"]) && $post_data["email"] == $data['email']) {
            $result['message'] = array('text' => l('error_emails_are_equal', 'users'), 'type' => View::MSG_ERROR);
        } elseif (isset($post_data["phone"]) && $post_data["phone"] == $data['phone']) {
            $result['message'] = array('text' => l('error_phones_are_equal', 'users'), 'type' => View::MSG_ERROR);
        } else {
            if ($validate_data["data"]["email"]) {
                $validate_data["data"]["email_restore"] = $data['email'];
                $validate_data["data"]["email_restore_code"] = substr($this->Users_model->encode_confirm(date("Y-m-d H:i:s")
                                                                                                . $data["email"]), 0, 10);
                $result['email'] = $validate_data["data"]["email"];
            }
            if ($validate_data["data"]["phone"]) {
                $validate_data["data"]["phone_restore"] = $data['phone'];
                $validate_data["data"]["phone_restore_code"] = substr($this->Users_model->encode_confirm(date("Y-m-d H:i:s")
                                                                                                . $data["phone"]), 0, 10);
                $result['phone'] = $validate_data["data"]["phone"];
            }
            $this->Users_model->save_user($user_id, $validate_data["data"]);
            $result['success'] = true;

            $this->load->model("users/models/Auth_model");
            $this->Auth_model->update_user_session_data($user_id);

            // send notification
            $this->load->model("Notifications_model");
            $this->Users_model->set_format_settings('get_safe', false);
            $this->Users_model->set_format_settings('get_safe', true);

            if ($validate_data["data"]["email"]) {
                $data['restore_link'] = site_url() . "users/email_restore/" . $validate_data["data"]["email_restore_code"];
                $return = $this->Notifications_model->send_notification($data["email"], "users_email_restore",
                                                                        $data, '', $data["lang_id"]);
            }
            if ($validate_data["data"]["phone"]) {
                $data['restore_link'] = site_url() . "users/phone_restore/" . $validate_data["data"]["phone_restore_code"];
                $return = $this->Notifications_model->send_notification($data["email"], "phone_restore_code",
                                                                        $data, '', $data["lang_id"]);
            }
            if ($validate_data["data"]["password"]) {
                $return = $this->Notifications_model->send_notification($data["email"], "users_change_password",
                                                                        $data, '', $data["lang_id"]);
            }

            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_user_updated", "users"));
            $result['message'] = array('text' => l("success_user_updated", "users"), 'type' => View::MSG_SUCCESS);
        }

        $this->view->assign($result);
        $this->view->render();
    }

    /**
     * Change my registered data
     *
     * @return void
     */
    public function change_reg_data()
    {
        $user_id = $this->session->userdata("user_id");
        $this->Users_model->set_format_settings('get_safe', false);
        $data = $this->Users_model->get_user_by_id($user_id);
        $this->Users_model->set_format_settings('get_safe', true);

        if ($this->input->post("btn_passw_save")) {
            $post_data["password_old"] = $this->input->post("password_old", true);
            $post_data["password"] = $this->input->post("password", true);
            $post_data["repassword"] = $this->input->post("repassword", true);

            $validate_data = $this->Users_model->validate_user($user_id, $post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $this->Users_model->save_user($user_id, $validate_data["data"]);

                // send notification
                $this->load->model("Notifications_model");
                $this->Users_model->set_format_settings('get_safe', false);
                $user_data = $this->Users_model->get_user_by_id($user_id);
                $this->Users_model->set_format_settings('get_safe', true);
                $user_data["password"] = $post_data["password"];
                $return = $this->Notifications_model->send_notification($user_data["email"], "users_change_password",
                                                                        $user_data, '', $user_data["lang_id"]);

                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_user_updated", "users"));
                $this->view->setRedirect(site_url() . "users/change_reg_data");
            }
        }

        if ($this->input->post("btn_email_save")) {
            $post_data["email"] = $this->input->post("email", true);

            $validate_data = $this->Users_model->validate_user($user_id, $post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } elseif ($post_data["email"] == $data['email']) {
                $this->system_messages->addMessage(View::MSG_ERROR, l('error_emails_are_equal', 'users'));
            } else {
                if ($data['email']) {
                    $validate_data["data"]["email_restore"] = $data['email'];
                    $validate_data["data"]["email_restore_code"] = substr($this->Users_model->encode_confirm(date("Y-m-d H:i:s") . $data["email"]),
                                                                                                                  0, 10);
                }

                $this->Users_model->save_user($user_id, $validate_data["data"]);

                $this->load->model("users/models/Auth_model");
                $this->Auth_model->update_user_session_data($user_id);

                // send notification
                $this->load->model("Notifications_model");
                $this->Users_model->set_format_settings('get_safe', false);
                $user_data = $this->Users_model->get_user_by_id($user_id);
                $this->Users_model->set_format_settings('get_safe', true);
                $return = $this->Notifications_model->send_notification($user_data["email"], "users_change_email",
                                                                        $user_data, '', $user_data["lang_id"]);
                if ($data['email']) {
                    $user_data['restore_link'] = site_url() . "users/email_restore/" . $validate_data["data"]["email_restore_code"];
                    $return = $this->Notifications_model->send_notification($data["email"], "users_email_restore",
                                                                            $user_data, '', $data["lang_id"]);
                }

                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_user_updated", "users"));
                $this->view->setRedirect(site_url() . "users/change_reg_data");
            }
        }

        if ($this->input->post("btn_phone_save")) {
            $post_data["phone"] = $this->input->post("phone", true);

            $validate_data = $this->Users_model->validate_user($user_id, $post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } elseif ($post_data["phone"] == $data['phone']) {
                $this->system_messages->addMessage(View::MSG_ERROR, l('error_phones_are_equal', 'users'));
            } else {
                if ($data['phone']) {
                    $validate_data["data"]["phone_restore"] = $data['phone'];
                    $validate_data["data"]["phone_restore_code"] = substr($this->Users_model->encode_confirm(date("Y-m-d H:i:s") . $data["phone"]),
                                                                                                                  0, 10);
                }

                $this->Users_model->save_user($user_id, $validate_data["data"]);

                $this->load->model("users/models/Auth_model");
                $this->Auth_model->update_user_session_data($user_id);

                // send notification
                $this->load->model("Notifications_model");
                $this->Users_model->set_format_settings('get_safe', false);
                $user_data = $this->Users_model->get_user_by_id($user_id);
                $this->Users_model->set_format_settings('get_safe', true);
                $return = $this->Notifications_model->send_notification($user_data["email"], "users_change_phone",
                                                                        $user_data, '', $user_data["lang_id"]);
                if ($data['phone']) {
                    $user_data['restore_link'] = site_url() . "users/phone_restore/" . $validate_data["data"]["phone_restore_code"];
                    $return = $this->Notifications_model->send_notification($user_data["email"], "phone_restore_code",
                                                                            $user_data, '', $user_data["lang_id"]);
                }

                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_user_updated", "users"));
                $this->view->setRedirect(site_url() . "users/change_reg_data");
            }
        }

        $this->load->helper("start");
        $this->view->assign('phone_format', get_phone_format());

        // breadcrumbs
        $user_type = $this->session->userdata("user_type");
        $this->Menu_model->breadcrumbs_set_parent($user_type . "_my_profile_change_regdata_item");

        $this->view->assign("data", $data);
        $this->view->render("change_reg_data");
    }

    /**
     * Change my password
     *
     * @return void
     */
    public function change_password()
    {
        // breadcrumbs
        $this->Menu_model->breadcrumbs_set_active(l('text_change_password', 'users'));

        $this->view->render("change_password");
    }

    /**
     * Change my email
     *
     * @return void
     */
    public function change_email()
    {
        $user_id = $this->session->userdata("user_id");

        $this->Users_model->set_format_settings('get_safe', false);
        $data = $this->Users_model->get_user_by_id($user_id);
        $this->Users_model->set_format_settings('get_safe', true);

        if ($this->input->post("btn_save")) {
            $post_data["email"] = $this->input->post("email", true);

            $validate_data = $this->Users_model->validate_user($user_id, $post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } elseif ($post_data["email"] == $data['email']) {
                $this->system_messages->addMessage(View::MSG_ERROR, l('error_emails_are_equal', 'users'));
            } else {
                if ($data['email']) {
                    $validate_data["data"]["email_restore"] = $data['email'];
                    $validate_data["data"]["email_restore_code"] = substr($this->Users_model->encode_confirm(date("Y-m-d H:i:s") . $data["email"]),
                                                                                                                  0, 10);
                }

                $this->Users_model->save_user($user_id, $validate_data["data"]);

                $this->load->model("users/models/Auth_model");
                $this->Auth_model->update_user_session_data($user_id);

                // send notification
                $this->load->model("Notifications_model");
                $this->Users_model->set_format_settings('get_safe', false);
                $user_data = $this->Users_model->get_user_by_id($user_id);
                $this->Users_model->set_format_settings('get_safe', true);
                $return = $this->Notifications_model->send_notification($user_data["email"], "users_change_email",
                                                                        $user_data, '', $user_data["lang_id"]);
                if ($data['email']) {
                    $user_data['restore_link'] = site_url() . "users/email_restore/" . $validate_data["data"]["email_restore_code"];
                    $return = $this->Notifications_model->send_notification($data["email"], "users_email_restore",
                                                                            $user_data, '', $data["lang_id"]);
                }

                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_user_updated", "users"));
                $this->view->setRedirect(site_url() . "users/change_email");
            }
        }

        // breadcrumbs
        $this->Menu_model->breadcrumbs_set_parent($data['user_type'] . "_my_profile_change_regdata_item");

        $this->view->assign("data", $data);
        $this->view->render("change_email");
    }

    /**
     * Restore user email by code
     *
     * @param string $code email restore code
     *
     * @return void
     */
    public function email_restore($code)
    {
        $code = trim(strip_tags($code));
        if (empty($code)) {
            return show_404();
        }
        $this->Users_model->set_format_settings('get_safe', false);
        $user = $this->Users_model->get_user_by_email_restore_code($code);
        $this->Users_model->set_format_settings('get_safe', true);
        if (!$user) {
            $this->system_messages->addMessage(View::MSG_ERROR, l('error_email_restore_empty', 'users'));
        } else {
            $data['email'] = $user['email_restore'];
            $data['email_restore_code'] = '';
            $this->Users_model->save_user($user['id'], $data);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_user_updated", "users"));
        }
        $this->view->setRedirect(site_url());
    }

    /**
     * Restore user phone by code
     *
     * @param string $code phone restore code
     *
     * @return void
     */
    public function phone_restore($code)
    {
        $code = trim(strip_tags($code));
        if (empty($code)) {
            return show_404();
        }
        $this->Users_model->set_format_settings('get_safe', false);
        $user = $this->Users_model->get_user_by_phone_restore_code($code);
        $this->Users_model->set_format_settings('get_safe', true);
        if (!$user) {
            $this->system_messages->addMessage(View::MSG_ERROR, l('error_phone_restore_empty', 'users'));
        } else {
            $data['phone'] = $user['phone_restore'];
            $data['phone_restore_code'] = '';
            $this->Users_model->save_user($user['id'], $data);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_user_updated", "users"));
        }
        $this->view->setRedirect(site_url());
    }

    /**
     * Register user as private
     *
     * @return void
     */
    public function reg_private()
    {
        $this->register("private");
    }

    /**
     * Register user as company
     *
     * @return void
     */
    public function reg_company()
    {
        $this->register("company");
    }

    /**
     * Register user as agent
     *
     * @return void
     */
    public function reg_agent()
    {
        $this->register("agent");
    }

    /**
     * Register a new user
     *
     * @param string $user_type user type or group
     *
     * @return void
     */
    public function register($user_type)
    {
        if ($this->session->userdata("auth_type") == "user") {
            $this->view->setRedirect(site_url() . "users/profile");
        }

        if (!$this->pg_module->get_module_config("users", $user_type . "_register_enabled")) {
            show_404();
            return;
        }

        $data = array();

        if ($this->input->post("btn_register")) {
            $post_data = $this->input->post("data", true);
            $post_data["user_type"] = $user_type;

            $validate_data = $this->Users_model->validate_user(null, $post_data);

            if (!$this->input->post("license_agreement", true)) {
                $validate_data["errors"][] = l("error_license_agreement", "users");
            }

            $is_captcha_valid = $this->input->post("captcha_confirmation", true) == $this->session->userdata("captcha_word");
            if (!$is_captcha_valid) {
                $validate_data["errors"][] = l("error_invalid_captcha", "users");
            }

            $data = $validate_data["data"];
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $use_email_confirmation = $this->pg_module->get_module_config("users", $user_type . "_register_mail_confirm");
                $data["status"] = 1;
                if ($use_email_confirmation) {
                    $data["confirm"] = 0;
                    $data["confirm_code"] = substr($this->Users_model->encode_confirm(
                                    date("Y-m-d H:i:s") . $data["email"]), 0, 10
                    );
                } else {
                    $data["confirm"] = 1;
                    $data["confirm_code"] = "";
                }
                $data["lang_id"] = $this->session->userdata("lang_id");
                if (!$data["lang_id"]) {
                    $data["lang_id"] = $this->pg_language->get_default_lang_id();
                }

                $user_id = $this->Users_model->save_user(null, $data);

                // send notification
                $this->load->model("Notifications_model");
                $data["password"] = $post_data["password"];
                if ($use_email_confirmation) {
                    $link = site_url() . "users/confirm/" . $data["confirm_code"];
                    $data["confirm_block"] = str_replace("[link]", $link, l("confirm_block_text", "users"));
                }
                $this->Notifications_model->send_notification(
                        $data["email"], "users_registration", $data, '', $data["lang_id"]
                );

                if (!$use_email_confirmation) {
                    $this->load->model("users/models/Auth_model");
                    $auth_data = $this->Auth_model->login($user_id);
                    if (!empty($auth_data["errors"])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $auth_data["errors"]);
                    }
                    $this->view->setRedirect(site_url() . 'start/homepage');
                } else {
                    $this->system_messages->addMessage(View::MSG_INFO, l("info_please_checkout_mailbox", "users"));
                    $this->view->setRedirect();
                }
            }
        }

        $this->load->helper("start");
        $this->view->assign('phone_format', get_phone_format());

        $this->load->plugin("captcha");
        $this->config->load("captcha_settings");
        $captcha_settings = $this->config->item("captcha_settings");
        $captcha = create_captcha($captcha_settings);
        $this->session->set_userdata("captcha_word", $captcha["word"]);
        $data["captcha_image"] = $captcha["image"];
        $data["captcha_word_length"] = strlen($captcha["word"]);

        // breadcrumbs
        $this->Menu_model->breadcrumbs_set_active(l("header_register", "users"));

        if ($this->pg_module->get_module_config('country_identification', 'automatic_identification')) {
            $this->load->model('countries/models/Countries_model');
            $user_country = $this->Countries_model->get_country_by_id($this->session->userdata('id_country'));
            if (!empty($user_country) && empty($data['id_country'])) {
                $data['id_country'] = $user_country['code'];
            }
            $user_region = $this->Countries_model->get_region($this->session->userdata('id_region'));
            if (!empty($user_region) && empty($data['id_city'])) {
                $data['id_region'] = $user_region['id'];
            }
            $user_city = $this->Countries_model->get_city($this->session->userdata('id_city'));
            if (!empty($user_city) && empty($data['id_city'])) {
                $data['id_city'] = $user_city['id'];
            }
        }


        $this->view->assign('data', $data);
        $this->view->assign('user_type', $user_type);
        $this->view->assign('form', $this->getRegForm($user_type));
        $this->view->render('register');
    }

    private function getRegForm($user_type)
    {
        if('private' === $user_type && $this->pg_module->is_module_active('users_private')) {
            return $this->view->fetch($user_type . '_register', 'user', 'users_private');
        } else {
            return $this->view->fetch($user_type . '_register', 'user', 'users');
        }
    }

    /**
     * Restore user password
     *
     * @return void
     */
    public function restore()
    {
        if ($this->session->userdata("auth_type") == "user") {
            $this->view->setRedirect();
        }

        if ($this->input->post("btn_save")) {
            $email = strip_tags($this->input->post("email", true));
            $this->Users_model->set_format_settings('get_safe', false);
            $user_data = $this->Users_model->get_user_by_email($email);
            $this->Users_model->set_format_settings('get_safe', true);
            $error = "";
            if (empty($user_data) || !$user_data["id"]) {
                $this->system_messages->addMessage(View::MSG_ERROR, l("error_user_no_exists", "users"));
            } elseif (!$user_data["confirm"]) {
                $this->system_messages->addMessage(View::MSG_ERROR, l("error_unconfirmed_user", "users"));
            } elseif (!$user_data["status"]) {
                $this->system_messages->addMessage(View::MSG_ERROR, l("error_user_is_blocked", "users"));
            } else {
                $save_data["password"] = $save_data["repassword"] = $user_data['password'] = substr(md5(date("Y-m-d H:i:s") . $user_data["email"]),
                                                                                                             0, 6);
                $validate_data = $this->Users_model->validate_user($user_data["id"], $save_data);
                if (!empty($validate_data['errors'])) {
                    $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
                } else {
                    $this->Users_model->save_user($user_data["id"], $validate_data['data']);

                    $this->load->model("Notifications_model");
                    $this->Notifications_model->send_notification($user_data["email"], "users_fogot_password",
                                                                  $user_data, '', $user_data["lang_id"]);
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_restore_mail_sent", "users"));
                    $this->view->setRedirect(site_url() . "users/restore");
                }
            }
        }
        // breadcrumbs
        $this->Menu_model->breadcrumbs_set_active(l("header_restore_password", "users"));

        $this->view->render("forgot_form");
    }

    /**
     * Confirm registration by code from email
     *
     * @param string $code code from email
     *
     * @return void
     */
    public function confirm($code)
    {
        $code = trim(strip_tags($code));
        $this->Users_model->set_format_settings('get_safe', false);
        $user = $this->Users_model->get_user_by_confirm_code($code);
        $this->Users_model->set_format_settings('get_safe', true);
        if (empty($user)) {
            $this->system_messages->addMessage(View::MSG_ERROR, l("error_user_no_exists", "users"));
            $this->view->setRedirect();
        } elseif ($user["confirm"]) {
            $this->system_messages->addMessage(View::MSG_ERROR, l("error_user_already_confirm", "users"));
            $this->view->setRedirect();
        } else {
            $data["confirm"] = 1;
            $this->Users_model->save_user($user["id"], $data);

            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_confirm", "users"));

            $this->load->model("users/models/Auth_model");
            $auth_data = $this->Auth_model->login($user["id"]);
            if (!empty($auth_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $auth_data["errors"]);
                $this->view->setRedirect();
            } else {
                $this->view->setRedirect(site_url() . 'start/homepage');
            }
        }
    }

    /**
     * Get users by ajax
     *
     * @param integer $page page of results
     *
     * @return void
     */
    public function ajax_get_users($page = 1)
    {
        $selected = $this->input->post("selected", true);
        $user_type = $this->input->post("user_type", true);
        $filter = $this->input->post("filter", true);
        $search_string = trim(strip_tags($this->input->post("search", true)));
        $params = $return = array();
        $items_on_page = 100;

        if (!empty($search_string)) {
            $params["where_sql"][] = "(email LIKE '%" . $search_string . "%' OR fname LIKE '%" . $search_string . "%' OR sname LIKE '%" . $search_string . "%' OR company_name LIKE '%" . $search_string . "%')";
        }
        if (!empty($selected)) {
            $params["where_sql"][] = "id NOT IN (" . implode($selected) . ")";
        }
        if (!empty($filter)) {
            foreach ($filter as $key => $value) {
                if (is_array($value)) {
                    $params["where_in"][$key] = $value;
                } else {
                    $params["where"][$key] = $value;
                }
            }
        }
        if ($user_type) {
            $params["where"]["user_type"] = $user_type;
        }
        $return["all"] = $this->Users_model->get_users_count($params);
        $return["pages"] = ceil($return["all"] / $items_on_page);
        $return["current_page"] = $page;
        $return["items"] = $this->Users_model->get_users_list($page, $items_on_page, array("email" => "asc"), $params);

        $this->view->assign($return);
        $this->view->render();

        return;
    }

    /**
     * Get selected users by ajax
     *
     * @return void
     */
    public function ajax_get_selected_users()
    {
        $selected = $this->input->post('selected', true);
        if (!empty($selected)) {
            $params['where_in']['id'] = $selected;
            $return = $this->Users_model->get_users_list(
                    null, null, array('email' => 'asc'), $params
            );
        } else {
            $return = array();
        }
        $this->view->assign('selected_users', $return);
        $this->view->render();
        return;
    }

    /**
     * Display users selector by ajax
     *
     * @param integer $max_select max selected users
     * @param string  $template   form template
     *
     * @return void
     */
    public function ajax_get_users_form($max_select = 1, $template = "default")
    {
        $selected = $this->input->post("selected", true);

        if (!empty($selected)) {
            $data["selected"] = $this->Users_model->get_users_list(null, null, array("unique_name" => "asc"), array(),
                                                                   $selected, true);
        } else {
            $data["selected"] = array();
        }
        $data["max_select"] = $max_select ? $max_select : 0;

        $this->view->assign("select_data", $data);
        $this->view->render("ajax_user_select_form_" . $template);
    }

    /**
     * Activate user contact
     *
     * @param integer $id_contact_user contact identifier
     * @param integer $id_user_service service identifier
     *
     * @return void
     */
    public function activate_contact($id_contact_user, $id_user_service)
    {

    }

    /**
     * Return users block
     *
     * @param string  $user_type       user type
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return string
     */
    protected function _users_block($user_type, $order = "default", $order_direction = "DESC", $page = 1)
    {
        $current_settings = isset($_SESSION["users_list"]) ? $_SESSION["users_list"] : array();
        if (!isset($current_settings["order"])) {
            $current_settings["order"] = "default";
        }
        if (!isset($current_settings["order_direction"])) {
            $current_settings["order_direction"] = "DESC";
        }
        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }
        if (!isset($current_settings["data"])) {
            $current_settings["data"] = array();
        }

        if (!isset($current_settings['data']['type']) || $current_settings['data']['type'] != $user_type) {
            $current_settings['data']['type'] = $user_type;
            $current_settings['page'] = 1;
        }
        $this->view->assign('user_type', $user_type);

        $filters = $current_settings['data'];
        $filters['active'] = 1;

        if ($this->session->userdata("auth_type") == "user") {
            $filters['not_user'] = $this->session->userdata("user_id");
        }

        $order = trim(strip_tags($order));
        if (!empty($order)) {
            $current_settings['order'] = $order;
        }
        $this->view->assign('order', $order);
        $order = $current_settings['order'];

        $order_direction = strtoupper(trim(strip_tags($order_direction)));
        if (!empty($order_direction)) {
            $current_settings['order_direction'] = $order_direction;
        }
        $order_direction = $current_settings['order_direction'];
        $this->view->assign('order_direction', $order_direction);

        if (!empty($page)) {
            $current_settings['page'] = $page;
        }
        $page = $current_settings['page'];

        if ($order_direction != 'ASC') {
            $order_direction = 'DESC';
        }

        $_SESSION['users_pagination'] = array(
            'order' => $order,
            'order_direction' => $order_direction,
            'page' => $page,
            'data' => $filters,
        );

        $view_mode = isset($_SESSION['users_view_mode']) ? $_SESSION['users_view_mode'] : 'list';

        $items_on_page = $this->pg_module->get_module_config("start", "index_items_per_page");

        if (isset($filters['keyword']) && !empty($filters['keyword'])) {
            $method = 'search';
            $keyword = $filters['keyword'];
        } else {
            $method = 'index';
            $keyword = '';
        }

        $this->load->helper('seo');
        $action_data = array('user_type' => $user_type, 'keyword' => $keyword);
        $url = rewrite_link('users', $method, $action_data);

        $sort_data = array(
            "url" => $url,
            "order" => $order,
            "direction" => $order_direction,
            "links" => array(
                "name" => l("field_name", "users"),
                "default" => l("field_date_created", "users"),
            ),
        );
        if ($this->pg_module->is_module_active('reviews')) {
            $this->load->helper('reviews');
            $review_link = get_reviews_sorter();
            if (is_array($review_link)) {
                $sort_data['links'] = array_merge($review_link, $sort_data['links']);
            }
        }
        $this->view->assign('sort_data', $sort_data);

        $use_map_in_search = true;
        if ($user_type == 'company') {
            if ($view_mode == 'list') {
                $use_map_in_search = $this->pg_module->get_module_config('users', 'use_map_in_search');
                $this->view->assign('use_map_in_search', $use_map_in_search);
            }
            $this->view->assign('use_map', true);
        }

        $users_count = $this->Users_model->get_users_count_by_filters($filters);
        if ($users_count > 0) {
            $order_array = array();
            switch ($order) {
                case 'default':
                    $order_array = array('id' => $order_direction);
                    break;
                case 'name':
                    if ($user_type === 'company') {
                        $order_array = array('company_name' => $order_direction);
                    } else {
                        $order_array = array("CONCAT(fname, ' ', sname)" => $order_direction);
                    }
                    break;
                default:
                    $order_array = array($order => $order_direction);
                    break;
            }

            $this->load->library('mobile_detect');
            $is_mobile = $this->mobile_detect->isMobile();
            $this->view->assign('is_mobile', $is_mobile);

            if ($use_map_in_search) {
                $this->Users_model->set_format_settings('get_contact', true);
            }
            $users = $this->Users_model->get_users_list_by_filters($filters, $page, $items_on_page, $order_array);
            $this->Users_model->set_format_settings('get_contact', false);
            $this->view->assign('users', $users);
            if ($use_map_in_search) {
                $markers = array();
                foreach ($users as $user) {
                    if (!$user['is_map'] || !$user['is_contact'] || !$user['is_show_contact']) {
                        continue;
                    }
                    $this->view->assign('view_mode', $view_mode);
                    $this->view->assign('user', $user);
                    $this->view->assign("date_format", $this->pg_date->get_format('date_literal', 'st'));
                    $info = $this->view->fetch('user_map_block', 'user', 'users');
                    $markers[] = array(
                        'gid' => $user['id'],
                        'country' => $user['country'],
                        'region' => $user['region'],
                        'city' => $user['city'],
                        'address' => $user['address'],
                        'postal_code' => $user['postal_code'],
                        'lat' => (float) $user['lat'],
                        'lon' => (float) $user['lon'],
                        'info' => $info,
                        'dragging' => false,
                    );
                }
                $this->view->assign('markers', $markers);
            }
        }

        $current_language = $this->pg_language->get_lang_by_id($this->pg_language->current_lang_id);
        $view_settings = array(
            'lang' => $current_language['code'],
        );
        $this->view->assign('map_settings', $view_settings);

        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $users_count, $items_on_page);
        $current_settings['page'] = $page;

        $this->load->helper('navigation');

        $page_data = get_user_pages_data($url . '/' . $order . '/' . $order_direction . '/', $users_count,
                                         $items_on_page, $page, 'briefPage');
        $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
        $this->view->assign("page_data", $page_data);

        $_SESSION['users_list'] = $current_settings;

        $this->view->assign('view_mode', $view_mode);

        return $this->view->fetch('users_block_' . $view_mode, 'user', 'users');
    }

    /**
     * Catalogue of users
     *
     * @param string  $user_type       user type
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function index($user_type = null, $order = 'default', $order_direction = 'DESC', $page = 1)
    {
        $this->search($user_type, '', $order, $order_direction, $page);
    }

    /**
     * Catalogue of users by ajax
     *
     * @param string  $user_type       user type
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function ajax_users($user_type = null, $order = 'default', $order_direction = 'DESC', $page = 1)
    {
        $this->view->output($this->_users_block($user_type, $order, $order_direction, $page));

        $this->view->render();
    }

    /**
     * Found users after searching
     *
     * @param string  $user_type       user type
     * @param string  $keyword         keyword string
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function search($user_type = null, $keyword = '', $order = 'default', $order_direction = 'DESC', $page = 1)
    {
        $user_types = $user_types_default = $this->Users_model->getUserTypes();
        if (empty($user_types)) {
            show_404();
        }
        foreach ($user_types as $key => $value) {
            $search_enabled = $this->pg_module->get_module_config("users", $value . "_search_enabled");
            if (!$search_enabled) {
                unset($user_types[$key]);
            }
        }

        $search_disabled = false;
        if (empty($user_types)) {
            $search_disabled = true;
            $user_types = $user_types_default;
        }
        $this->view->assign('user_types', $user_types);

        if ($user_type && in_array($user_type, $user_types)) {
            $current_settings = isset($_SESSION['users_list']) ? $_SESSION['users_list'] : array();

            if (!isset($current_settings['data']['keyword'])) {
                $current_settings['data']['keyword'] = '';
            }

            $keyword = trim(strip_tags(urldecode($keyword)));
            if ($keyword != $current_settings['data']['keyword']) {
                $current_settings['order'] = 'default';
                $current_settings['order_direction'] = 'DESC';
                $current_settings['page'] = 1;
                $current_settings['data'] = array();
            }

            $current_settings['data']['type'] = $user_type;
            $current_settings['data']['keyword'] = $keyword;

            $_SESSION['users_list'] = $current_settings;

        } elseif (isset($_SESSION['users_list']['data']['type']) &&
                isset($user_types[$_SESSION['users_list']['data']['type']])) {
            $user_type = $_SESSION['users_list']['data']['type'];
        } else {
            $user_type = current($user_types);
        }

        $user_block = $this->_users_block($user_type, $order, $order_direction, $page);
        $this->view->assign("block", $user_block);

        $this->view->assign('current_user_type', $user_type);

        if ($search_disabled) {
            switch ($user_type) {
                case 'private':
                    $this->Menu_model->breadcrumbs_set_active(l('privates', 'users'));
                    break;
                case 'company':
                    $this->Menu_model->breadcrumbs_set_active(l('companies', 'users'));
                    break;
                case 'agent':
                    $this->Menu_model->breadcrumbs_set_active(l('agents', 'users'));
                    break;
            }
        } else {
            $this->Menu_model->breadcrumbs_set_parent('main-menu-pro-' . $user_type . '-item');
        }

        $use_poll_in_search = $this->pg_module->get_module_config('users', 'use_poll_in_search');
        $this->view->assign('use_poll_in_search', $use_poll_in_search);

        $current_settings = isset($_SESSION["users_list"]) ? $_SESSION["users_list"] : array();
        if (!isset($current_settings["order"])) {
            $current_settings["order"] = "default";
        }
        $this->view->assign('order', $current_settings["order"]);

        if (!isset($current_settings["order_direction"])) {
            $current_settings["order_direction"] = "DESC";
        }
        $this->view->assign('order_direction', $current_settings["order_direction"]);

        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }
        $this->view->assign('page', $current_settings["page"]);

        $this->pg_seo->set_seo_data(array('user_type' => $user_type, 'user_type_str' => l($user_type, 'users')));

        $view_mode = isset($_SESSION['users_view_mode']) ? $_SESSION['users_view_mode'] : 'list';

        $this->view->render('users_' . $view_mode, 'user', 'users');
    }

    /**
     * Found users after searching by ajax
     *
     * @param string  $user_type       user type
     * @param string  $filters         keyword string
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function ajax_search()
    {
        $current_settings = isset($_SESSION['users_list']) ? $_SESSION['users_list'] : array();
        $current_settings["order"] = "default";
        $current_settings["order_direction"] = "DESC";
        $current_settings["page"] = 1;
        $current_settings["data"] = array();

        $user_type = $this->input->post("user_type");
        if ($user_type) {
            $current_settings['data']['type'] = $user_type;
        }

        $country = strval($this->input->post("id_country"));
        if ($country) {
            $current_settings["data"]["country"] = $country;
        }

        $region = intval($this->input->post("id_region"));
        if ($region) {
            $current_settings["data"]["region"] = $region;
        }

        $city = intval($this->input->post("id_city"));
        if ($city) {
            $current_settings["data"]["city"] = $city;
        }

        $district = intval($this->input->post("id_district"));
        if ($district) {
            $current_settings["data"]["district"] = $district;
        }

        $zip = trim(strip_tags($this->input->post("zip")));
        if (!empty($zip)) {
            $current_settings["data"]["zip"] = intval($zip);
        }

        if ($current_settings['data']['type'] != 'company') {
            if (isset($current_settings["data"]["country"])) {
                unset($current_settings["data"]["country"]);
            }
            if (isset($current_settings["data"]["region"])) {
                unset($current_settings["data"]["region"]);
            }
            if (isset($current_settings["data"]["city"])) {
                unset($current_settings["data"]["city"]);
            }
        }

        $keyword = trim(strip_tags($this->input->post("keyword")));
        if (!empty($keyword)) {
            $current_settings["data"]["keyword"] = $keyword;
        }

        // field editor
        $form = $this->input->post('form', true);
        if ($form) {
            $data['field_editor_data'][$form] = $this->input->post('data', true);

            if (!is_array($data['field_editor_data'][$form])) {
                $data['field_editor_data'][$form] = array();
            }
            $field_editor_range = $this->input->post('field_editor_range', true);
            if (!is_array($field_editor_range)) {
                $field_editor_range = array();
            }
            $short_value = $this->input->post('short_value', true);

            foreach ($data['field_editor_data'][$form] as $field_gid => $field_value) {
                if (isset($field_value['range']['min'])) {
                    if (isset($field_editor_range[$field_gid]['min'])) {
                        $min_value = $field_editor_range[$field_gid]['min'];
                    } else {
                        $min_value = 0;
                    }
                    if ($field_value['range']['min'] <= $min_value) {
                        unset($data['field_editor_data'][$form][$field_gid]['range']['min']);
                    }
                }
                if (isset($field_value['range']['max'])) {
                    if (isset($field_editor_range[$field_gid]['max'])) {
                        $max_value = $field_editor_range[$field_gid]['max'];
                        if ($field_value['range']['max'] >= $max_value) {
                            unset($data['field_editor_data'][$form][$field_gid]['range']['max']);
                        }
                    } elseif (empty($field_value['range']['max'])) {
                        unset($data['field_editor_data'][$form][$field_gid]['range']['max']);
                    }
                }
                if (isset($data['field_editor_data'][$form][$field_gid]['range']) &&
                        empty($data['field_editor_data'][$form][$field_gid]['range'])) {
                    unset($data['field_editor_data'][$field_gid]['range']);
                }
            }
            if (isset($current_settings['data']['field_editor_data'])) {
                foreach ($current_settings['data']['field_editor_data'] as $form_name => $form_data) {
                    if ($form_name == $form) {
                        continue;
                    }
                    $data['field_editor_data'][$form_name] = array_diff_key($form_data,
                                                                            $data['field_editor_data'][$form]);
                    if (empty($data['field_editor_data'][$form_name])) {
                        unset($data['field_editor_data'][$form_name]);
                    }
                }
            }
            foreach ($data['field_editor_data'][$form] as $field_gid => $field_data) {
                if (empty($field_data)) {
                    unset($data['field_editor_data'][$form][$field_gid]);
                } elseif (is_array($field_data)) {
                    $empty = true;
                    foreach ($field_data as $value) {
                        if (!empty($value)) {
                            $empty = false;
                            break;
                        }
                    }
                    if ($empty) {
                        unset($data['field_editor_data'][$form][$field_gid]);
                    }
                }
            }
            if (empty($data['field_editor_data'][$form])) {
                unset($data['field_editor_data'][$form]);
            }
        }
        if (!empty($data)) {
            foreach ($data as $key => $value) {
                if (!empty($value)) {
                    $current_settings['data'][$key] = $value;
                } elseif (isset($current_settings['data'][$key])) {
                    unset($current_settings['data'][$key]);
                }
            }
        }

        $_SESSION['users_list'] = $current_settings;

        $this->load->helper('seo');

        $search_action_data = $this->Users_model->get_search_link_data();
        $this->view->output(rewrite_link('users', $search_action_data['keyword'] ? 'search' : 'index',
                                         $search_action_data));

        $this->view->render();
    }

    /**
     * Display user profile
     *
     * @param integer $user_id     user identifier
     * @param string  $section_gid section guid
     * @param boolean $pdf         generate pdf version
     *
     * @return void
     */
    public function view($user_id, $section_gid = "contacts", $pdf = false)
    {
        if (!$user_id) {
            show_404();
            return;
        }

        if (!isset($this->Users_model->display_sections[$section_gid])) {
            $section_gid = 'contacts';
        }

        $format_users = array('get_contact', 'get_company', 'get_views');
        switch ($section_gid) {
            case 'contacts':
                $format_users[] = 'get_field_editor';
                break;
        }

        $this->Users_model->set_format_settings(array_fill_keys($format_users, true));
        $user = $this->Users_model->get_user_by_id($user_id);
        $this->Users_model->set_format_settings(array_fill_keys($format_users, false));
        if (!$user) {
            show_404();
            return;
        }

        $display_sections = $this->Users_model->get_display_sections($user);
        if ($this->session->userdata('auth_type') != 'user' || $this->session->userdata('user_id') != $user_id) {
            if (!$user['status']) {
                show_404();
                return;
            }
        }

        foreach ($display_sections as $display_section_gid => $used) {
            if (!$used) {
                continue;
            }
            $user_content[$display_section_gid] = $this->_get_section_html($user, $display_section_gid);
        }

        $this->view->assign("display_sections", $display_sections);
        $this->view->assign("user_content", $user_content);
        $this->view->assign("section_gid", $section_gid);

        //pagination
        $page_data = array();
        $page_data["date_format"] = $this->pg_date->get_format('date_literal', 'st');
        $this->view->assign("page_data", $page_data);

        $is_owner = 0;
        if ($this->session->userdata("auth_type") == "user") {
            $auth_id = $this->session->userdata("user_id");
            $this->view->assign('user_id', $auth_id);
            if ($auth_id == $user_id) {
                $is_owner = 1;
            }
        }
        $this->view->assign('is_user_owner', $is_owner);

        $this->load->model('users/models/Users_visitor_model');
        $this->Users_visitor_model->check_views_count($user_id);

        $this->Menu_model->breadcrumbs_set_parent("main-menu-pro-item");
        $this->Menu_model->breadcrumbs_set_active($user["output_name"], site_url() . "users/view/" . $user_id);

        if ($pdf) {
            $this->pg_theme->print_type = "pdf";
            $tpl_output = $this->view->fetch("view");
            $this->Users_model->get_pdf_file($tpl_output, $user);
        } else {
            $this->load->helper('seo');

            $seo_data = array(
                'id' => $user['id'],
                'user_type' => $user['user_type'],
                'name' => $user['name'],
                'section' => 'contacts',
            );

            $lang_canonical = true;

            if ($this->pg_module->is_module_active('seo')) {
                $lang_canonical = $this->pg_module->get_module_config('seo', 'lang_canonical');
            }

            if ($user['id_seo_settings']) {
                $this->load->model('Seo_advanced_model');
                $seo_settings = $this->Seo_advanced_model->parse_seo_tags($user['id_seo_settings']);
                $seo_settings['canonical'] = rewrite_link('users', 'view', $seo_data, false, null, $lang_canonical);
                $seo_settings['image'] = $user['media']['user_logo']['thumbs']['big'];
                $this->pg_seo->set_seo_tags($seo_settings);
            } else {
                $seo_settings = $user;
                $seo_settings['canonical'] = rewrite_link('users', 'view', $seo_data, false, null, $lang_canonical);
                $seo_settings['image'] = $user['media']['user_logo']['thumbs']['big'];
                $seo_settings['section'] = $section_gid;
                $seo_settings['name'] = '<span itemprop="name">' . $user['name'] . '</span>';

                $location_data = array(
                    array(
                        'country' => $user['id_country'],
                        'region' => $user['id_region'],
                        'city' => $user['id_city'],
                        'district' => $user['id_district'],
                        'address' => $user['address'],
                        'zip_code' => $user['postal_code'],
                    ),
                );
                $this->load->helper('countries');
                $user_locations = address_output_format($location_data, 'city', true);
                $seo_settings['location'] = $user_locations[0];

                $this->pg_seo->set_seo_data($seo_settings);
            }

            $this->view->assign('user_contact_message', l('text_user_message', 'users'));
            $this->view->render("view", "user", "users");
        }
    }

    /**
     * Get section content of user profile
     *
     * @param array  $data        user data
     * @param string $section_gid section guid
     *
     * @return string
     */
    protected function _get_section_html($data, $section_gid = "contacts")
    {
        $this->load->helper('users');
        $this->view->assign("section_gid", $section_gid);
        $this->view->assign("user", $data);
        switch ($section_gid) {
            case 'overview':
                $date_format = $this->pg_date->get_format('date_literal', 'st');
                $this->view->assign("date_format", $date_format);
                break;
            case 'map':
            case 'map_info':
                $markers = array(
                    array(
                        'gid' => $data['id'],
                        'country' => $data['country'],
                        'region' => $data['region'],
                        'city' => $data['city'],
                        'address' => $data['address'],
                        'postal_code' => $data['postal_code'],
                        'lat' => (float) $data['lat'],
                        'lon' => (float) $data['lon'],
                        'info' => ($data['location'] ? $data['location'] : $data['output_name'] . ', ' . l($data['user_type'],
                                                                                                           'users')),
                        'dragging' => false,
                    ),
                );
                $this->view->assign("markers", $markers);

                $current_language = $this->pg_language->get_lang_by_id($this->pg_language->current_lang_id);
                $view_settings = array(
                    'lang' => $current_language['code'],
                );
                $this->view->assign('map_settings', $view_settings);
                break;
        }

        return $this->view->fetch('view_content', 'user', 'users');
    }

    /**
     * Display section content of profile by ajax
     *
     * @param integer $user_id     user identifier
     * @param string  $section_gid section guid
     * @param string  $template    contacts section template
     *
     * @return void
     */
    public function ajax_get_section($user_id, $section_gid, $template = '')
    {
        $this->Users_model->set_format_settings('get_contact', true);
        $user = $this->Users_model->get_user_by_id($user_id);
        $this->Users_model->set_format_settings('get_contact', false);
        $this->view->assign('ajax', 1);
        $this->view->assign('template', $template);

        $this->view->output($this->_get_section_html($user, $section_gid));

        $this->view->render();
    }

    /**
     * Return my agents block
     *
     * @param integer $action          action name
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    protected function _my_agents_block($action = "list", $order = null, $order_direction = null, $page = 1)
    {
        $company_id = $this->session->userdata("user_id");

        switch ($action) {
            case "requests":
                $agent_status = 0;
                break;
            case "list":
            default:
                $agent_status = 1;
                $action = "list";
                break;
        }

        $current_settings = isset($_SESSION["users_agents_list"]) ? $_SESSION["users_agents_list"] : array();
        if (!isset($current_settings["order"])) {
            $current_settings["order"] = "id";
        }
        if (!isset($current_settings["order_direction"])) {
            $current_settings["order_direction"] = "DESC";
        }
        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }

        if (!isset($current_settings["name"])) {
            $current_settings["name"] = "";
        }

        $name = null;

        if (!empty($_POST)) {
            $current_settings["name"] = $this->input->post('name', true);
        }

        if (!empty($current_settings["name"])) {
            $name = $current_settings["name"];
        }

        $current_settings["filter"] = $agent_status;
        $current_settings["page"] = $page;

        if (!$order) {
            $order = $current_settings["order"];
        }
        $this->view->assign("order", $order);
        $current_settings["order"] = $order;

        if (!$order_direction) {
            $order_direction = $current_settings["order_direction"];
        }
        $this->view->assign("order_direction", $order_direction);
        $current_settings["order_direction"] = $order_direction;

        $agents_count = $this->Users_model->getAgentsCount($company_id, $agent_status, $name);

        if ($agent_status) {
            $this->view->assign('agents_count', $agents_count);
            $this->view->assign('requests_count', $this->Users_model->getAgentsCount($company_id, 0, $name));
        } else {
            $this->view->assign('agents_count', $this->Users_model->getAgentsCount($company_id, 1, $name));
            $this->view->assign('requests_count', $agents_count);
        }

        if (!$page) {
            $page = $current_settings["page"];
        }
        $items_on_page = $this->pg_module->get_module_config("start", "index_items_per_page");

        $this->load->helper("sort_order");
        $page = get_exists_page_number($page, $agents_count, $items_on_page);
        $current_settings["page"] = $page;

        $_SESSION["users_agents_list"] = $current_settings;

        $build_link = function($field) use ($action, $order, $order_direction) {
            return site_url() . "users/my_agents/$action/$field/" .
                (($order !== $field xor $order_direction === "DESC") ? "ASC" : "DESC");
        };

        $sort_links = array(
            "name" => $build_link('name'),
            "email" => $build_link('email'),
            "phone" => $build_link('phone'),
            "agent_date" => $build_link('agent_date'),
        );
        $this->view->assign("sort_links", $sort_links);

        switch ($order) {
            case 'name':
                $order = "CONCAT(fname, ' ', sname)";
                break;
            case 'email':
                $order = 'contact_email';
                break;
            case 'phone':
                $order = 'contact_phone';
                break;
            case 'agent_date':
                $order = 'agent_date';
                break;
            default:
                $order = 'name';
                break;
        }

        if ($agents_count > 0) {
            $agents = $this->Users_model->getAgentsList($company_id, $agent_status, $page, $items_on_page,
                                                        array($order => $order_direction), true, $name);
            $this->view->assign("agents", $agents);
        }

        $this->view->assign("agent_status", $agent_status);
        $this->view->assign("action", $action);

        $this->load->helper("navigation");

        $url = site_url() . "users/my_agents/" . $action . "/" . $order . "/" . $order_direction . "/";
        $page_data = get_user_pages_data($url, $agents_count, $items_on_page, $page, "briefPage");
        $page_data["date_format"] = $this->pg_date->get_format('date_time_literal', 'st');
        $page_data['order'] = $current_settings['order'];
        $page_data['order_direction'] = $current_settings['order_direction'];
        $page_data['name'] = $current_settings["name"];
        $this->view->assign("page_data", $page_data);

        return $this->view->fetch("my_agents_block", "user", "users");
    }

    /**
     * Manage my agents
     *
     * @param integer $action          action name
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function my_agents($action = null, $order = null, $order_direction = null, $page = null)
    {
        $user_type = $this->session->userdata("user_type");

        if ($user_type != 'company') {
            show_404();
        }

        $current_settings = isset($_SESSION["my_agents_search"]) ? $_SESSION["my_agents_search"] : array();

        if (empty($current_settings["action"])) {
            $current_settings["action"] = 'list';
        }
        if (empty($current_settings['order'])) {
            $current_settings["order"] = 'date_modified';
        }
        if (empty($order_direction)) {
            $current_settings["order_direction"] = 'DESC';
        }
        if (empty($current_settings["page"])) {
            $current_settings["page"] = 1;
        }

        if ($action) {
            $current_settings["action"] = $action;
        }
        if ($order) {
            $current_settings["order"] = $order;
        }
        if ($order_direction) {
            $current_settings["order_direction"] = $order_direction;
        }
        if ($page) {
            $current_settings["page"] = $page;
        }
        $_SESSION["my_agents_search"] = $current_settings;

        $content = $this->_my_agents_block($current_settings["action"], $current_settings["order"],
                                           $current_settings["order_direction"], $current_settings["page"]);
        $this->view->assign("block", $content);

        $this->Menu_model->set_menu_active_item("company_account_menu",
                                                "company_my_agents_" . $current_settings["action"] . "_item");
        $this->Menu_model->breadcrumbs_set_parent("company_my_agents_" . $current_settings["action"] . "_item");

        $this->view->render("my_agents_list");
    }

    /**
     * Load my agents by ajax
     *
     * @param integer $action          action name
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function ajax_my_agents($action = "list", $order = "date_modified", $order_direction = "DESC", $page = 1)
    {
        $current_settings = isset($_SESSION["my_gents"]) ? $_SESSION["my_agents_search"] : array();
        $current_settings["action"] = $action;
        $current_settings["order"] = $order;
        $current_settings["order_direction"] = $order_direction;
        $current_settings["page"] = $page;
        $_SESSION["my_agents_search"] = $current_settings;

        $this->view->output($this->_my_agents_block($action, $order, $order_direction, $page));

        $this->view->render();
    }

    /**
     * Process request from my agents
     *
     * @param string  $action   action name
     * @param integer $agent_id agent identifier
     *
     * @return void
     */
    public function my_agents_request($action = "approve", $agent_id = null)
    {
        $errors = false;
        $messages = array();
        if (!$agent_id) {
            $agent_id = $this->input->post("ids");
        }
        if (!empty($agent_id)) {
            $company_id = $this->session->userdata("user_id");
            foreach ((array) $agent_id as $i) {
                $error = $this->Users_model->{$action . "_agent"}($company_id, $i);
                if ($error) {
                    $errors = true;
                    $messages[] = $error;
                } else {
                    $messages[] = l("success_" . $action . "_agent", "users");
                }
            }
            if ($errors) {
                $this->system_messages->addMessage(View::MSG_ERROR, $messages);
            } else {
                $this->system_messages->addMessage(View::MSG_SUCCESS, $messages);
            }
        }

        $this->view->setRedirect(site_url() . "users/my_agents/" . ($action == 'approve' ? 'list' : 'requests'));
    }

    /**
     * Remove selected agents from me
     *
     * @param integer $agent_id agent identifier
     *
     * @return void
     */
    public function my_agents_delete($agent_id = null)
    {
        $errors = false;
        $messages = array();
        if (!$agent_id) {
            $agent_id = $this->input->post("ids");
        }
        if (!empty($agent_id)) {
            $company_id = $this->session->userdata("user_id");
            foreach ((array) $agent_id as $i) {
                $error = $this->Users_model->delete_agent($company_id, $i);
                if ($error) {
                    $errors = true;
                    $messages[] = $error;
                } else {
                    $messages[] = l("success_deleted_agent", "users");
                }
            }
            if ($errors) {
                $this->system_messages->addMessage(View::MSG_ERROR, $messages);
            } else {
                $this->system_messages->addMessage(View::MSG_SUCCESS, $messages);
            }
        }
        $this->view->setRedirect(site_url() . "users/my_agents/list");
    }

    /**
     * I get out from company
     *
     * @return void
     */
    public function agent_get_out()
    {
        $user_id = $this->session->userdata("user_id");
        $user = $this->Users_model->get_user_by_id($user_id);
        $error = $this->Users_model->delete_agent($user["agent_company"], $user_id, true);
        if ($error) {
            $this->system_messages->addMessage(View::MSG_ERROR, $error);
        } else {
            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_agent_get_out", "users"));
        }
        $url = site_url() . "users/profile";
        $this->view->setRedirect($url);
    }

    /**
     * Change my currency
     *
     * @param integer $currency_id currency identifier
     *
     * @return void
     */
    public function change_currency($currency_id)
    {
        $currency_id = intval($currency_id);
        $this->session->set_userdata("currency_id", $currency_id);

        if ($this->session->userdata("auth_type") == "user") {
            $user_id = $this->session->userdata("user_id");
            $save_data["id_currency"] = $currency_id;
            $this->Users_model->save_user($user_id, $save_data);
        }

        if (strpos($_SERVER["HTTP_REFERER"], site_url()) !== false) {
            $this->view->setRedirect($_SERVER["HTTP_REFERER"]);
        } else {
            $this->view->setRedirect();
        }
    }

    /**
     * Change my currency by ajax
     *
     * @param integer $currency_id currency identifier
     *
     * @return void
     */
    public function ajax_change_currency($currency_id)
    {
        $currency_id = intval($currency_id);
        $this->session->set_userdata("currency_id", $currency_id);

        if ($this->session->userdata("auth_type") == "user") {
            $user_id = $this->session->userdata("user_id");
            $save_data["id_currency"] = $currency_id;
            $this->Users_model->save_user($user_id, $save_data);
        }
    }

    /**
     * Set settings of user map
     *
     * @param array $param map settings
     *
     * @return void
     */
    public function ajax_set_map($param)
    {
        if ($this->session->userdata("auth_type") != "user") {
            return;
        }
        $user_id = $this->session->userdata("user_id");

        $user = $this->Users_model->get_user_by_id($user_id);
        if (!$user) {
            return;
        }

        $this->load->model('geomap/models/Geomap_model');
        $this->load->model('geomap/models/Geomap_settings_model');

        switch ($param) {
            case 'type': $settings['view_type'] = $this->input->post('type', true);
                break;
            case 'zoom': $settings['zoom'] = $this->input->post('zoom', true);
                break;
            case 'coordinates':
                $settings['lat'] = $this->input->post('lat', true);
                $settings['lon'] = $this->input->post('lon', true);
                break;
        }
        if (!$settings) {
            return;
        }
        $validate_data = $this->Geomap_settings_model->validate_settings($settings);
        if (empty($validate_data["errors"])) {
            $map_gid = $this->Geomap_model->get_default_driver_gid();
            $this->Geomap_settings_model->save_settings($map_gid, 0, $user_id, 'user_profile', $validate_data["data"]);
        }
        if ($param == 'coordinates') {
            $data['lat'] = $this->input->post('lat', true);
            $data['lon'] = $this->input->post('lon', true);
            $this->Users_model->save_user($user_id, $data);
        }

        return;
    }

    /**
     * Set coordinates of user map after geocoding
     *
     * @param integer $user_id user identifier
     *
     * @return void
     */
    public function ajax_set_geocode_coordinates($user_id)
    {
        $user = $this->Users_model->get_user_by_id($user_id);

        if (!$user) {
            return;
        }

        if ((float) $user['lat'] > 0 || (float) $user['lot'] > 0) {
            return;
        }

        $data['lat'] = $this->input->post('lat');
        $data['lon'] = $this->input->post('lon');

        $validate_data = $this->Users_model->validate_user($data);
        if (!empty($validate_data["errors"])) {
            $this->view->assign('error', implode('<br>', $validate_data["errors"]));

            $this->view->render();

            return;
        } else {
            $this->Users_model->save_user($user_id, $validate_data['data']);
        }

        $this->view->render();
    }

    /**
     * Update settings of user map by ajax
     *
     * @return void
     */
    public function ajax_update_map()
    {
        $user_id = $this->session->userdata("user_id");
        $data = $this->Users_model->get_user_by_id($user_id);

        $lat = $this->input->post('lat', true);
        $lon = $this->input->post('lon', true);

        $markers = array(
            array(
                "lat" => (float) $lat,
                "lon" => (float) $lon,
                "info" => $data["output_name"] . ', ' . l($data['user_type'], 'users'),
                "dragging" => true,
            ),
        );

        $this->load->helper('geomap');

        $this->view->output(update_default_map(array('markers' => $markers, 'rand' => 1)));

        $this->view->render();
    }

    /**
     * Get visitors/visits of user profile block
     *
     * @param string  $field_name      visitor/visit
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    protected function _visitors_block($field_name, $order = "id", $order_direction = "DESC", $page = 1)
    {
        $current_settings = isset($_SESSION["users_visitors_list"]) ? $_SESSION["users_visitors_list"] : array();
        if (!isset($current_settings["order"])) {
            $current_settings["order"] = "id";
        }
        if (!isset($current_settings["order_direction"])) {
            $current_settings["order_direction"] = "DESC";
        }
        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }
        if (!isset($current_settings["filters"])) {
            $current_settings["filters"] = array();
        }

        $visit_type = $field_name == 'id_visitor' ? 'visitors' : 'visits';

        $filters = $current_settings["filters"];

        if (!$order) {
            $order = $current_settings["order"];
        }
        $this->view->assign("order", $order);
        $current_settings["order"] = $order;

        if (!$order_direction) {
            $order_direction = $current_settings["order_direction"];
        }
        $this->view->assign("order_direction", $order_direction);
        $current_settings["order_direction"] = $order_direction;

        if (!$page) {
            $page = $current_settings["page"];
        }
        $items_on_page = $this->pg_module->get_module_config("start", "index_items_per_page");

        $sort_data = array(
            "url" => site_url() . "users/" . $visit_type,
            "order" => $order,
            "direction" => $order_direction,
            "links" => array(
                "visits_count" => l("field_visits_count", "users"),
                "last_visit_date" => l("field_last_visit_date", "users"),
            ),
        );
        $this->view->assign("sort_data", $sort_data);

        $this->load->model('users/models/Users_visitor_model');

        $users_count = $this->Users_visitor_model->get_visitors_count($filters);
        if ($users_count > 0) {
            $order_array = array($order => $order_direction);
            $visitors = $this->Users_visitor_model->get_visitors_list($filters, $page, $items_on_page, $order_array);
            $users_for_search = array();
            foreach ($visitors as $value) {
                $users_for_search[] = $value[$field_name];
            }
            $users = array();
            $results = $this->Users_model->get_users_list_by_key(null, null, null, array(), $users_for_search);
            foreach ($users_for_search as $user_id) {
                $users[$user_id] = isset($results[$user_id]) ? $results[$user_id] :
                        $this->Users_model->format_default_user($user_id);
            }
            $this->view->assign("users", $users);

            $users_for_contact = array();
            if ($this->pg_module->is_module_active("users_services")) {
                if ($this->session->userdata("auth_type") == "user") {
                    $auth_id = $this->session->userdata("user_id");

                    $this->load->model("Users_services_model");
                    foreach ($visitors as $value) {
                        if ($auth_id == $value[$field_name]) {
                            $users_for_contact[$value[$field_name]] = true;
                        } else {
                            $users_for_contact[$value[$field_name]] = $this->Users_services_model->is_available_contact($auth_id,
                                                                                                                        $value[$field_name]);
                        }
                    }
                    $this->view->assign('users_for_contact', $users_for_contact);
                }
            } else {
                $this->view->assign('is_contact', true);
            }
        }
        $this->view->assign('users_count', $users_count);

        $this->load->helper("sort_order");
        $page = get_exists_page_number($page, $users_count, $items_on_page);
        $current_settings["page"] = $page;

        $this->load->helper('navigation');

        $url = site_url() . 'users/' . $visit_type . '/' . $order . '/' . $order_direction . '/';
        $page_data = get_user_pages_data($url, $users_count, $items_on_page, $page, "briefPage");
        $page_data["date_format"] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign("page_data", $page_data);

        $_SESSION["users_visitors_list"] = $current_settings;

        return $this->view->fetch("visitors_block", "user", "users");
    }

    /**
     * Display visitors of user profile
     *
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function visitors($order = "id", $order_direction = "DESC", $page = 1)
    {
        $current_settings = isset($_SESSION["users_visitors_list"]) ? $_SESSION["users_visitors_list"] : array();

        $user_id = intval($this->session->userdata("user_id"));
        $filters = array('user' => $user_id);

        $current_settings["order"] = $order;
        $current_settings["order_direction"] = $order_direction;
        $current_settings["page"] = $page;
        $current_settings["filters"] = $filters;
        $_SESSION["users_visitors_list"] = $current_settings;

        $user_block = $this->_visitors_block('id_visitor', $order, $order_direction, $page);
        $this->view->assign("block", $user_block);

        $current_settings = isset($_SESSION["users_visitors_list"]) ? $_SESSION["users_visitors_list"] : array();
        if (!isset($current_settings["order"])) {
            $current_settings["order"] = "date_created";
        }
        $this->view->assign('order', $current_settings["order"]);

        if (!isset($current_settings["order_direction"])) {
            $current_settings["order_direction"] = "DESC";
        }
        $this->view->assign('order_direction', $current_settings["order_direction"]);

        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }
        $this->view->assign('page', $current_settings["page"]);

        $this->Menu_model->breadcrumbs_set_active(l("text_profile_visitors", "users"));

        $this->view->render("visitors", "user", "users");
    }

    /**
     * Display visitors of user profile by ajax
     *
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function ajax_visitors($order = "id", $order_direction = "DESC", $page = 1)
    {
        $current_settings = isset($_SESSION["users_visitors_list"]) ? $_SESSION["users_visitors_list"] : array();

        $user_id = intval($this->session->userdata("user_id"));
        $filters = array('user' => $user_id);

        $current_settings["order"] = $order;
        $current_settings["order_direction"] = $order_direction;
        $current_settings["page"] = $page;
        $current_settings["filters"] = $filters;
        $_SESSION["users_visitors_list"] = $current_settings;

        $this->view->output($this->_visitors_block('id_visitor', $order, $order_direction, $page));

        $this->view->render();
    }

    /**
     * Display visits of user profile
     *
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function visits($order = "id", $order_direction = "DESC", $page = 1)
    {
        $current_settings = isset($_SESSION["users_visitors_list"]) ? $_SESSION["users_visitors_list"] : array();

        $user_id = intval($this->session->userdata("user_id"));
        $filters = array('visitor' => $user_id);

        $current_settings["order"] = $order;
        $current_settings["order_direction"] = $order_direction;
        $current_settings["page"] = $page;
        $current_settings["filters"] = $filters;
        $_SESSION["users_visitors_list"] = $current_settings;

        $user_block = $this->_visitors_block('id_user', $order, $order_direction, $page);
        $this->view->assign("block", $user_block);

        $current_settings = isset($_SESSION["users_visitors_list"]) ? $_SESSION["users_visitors_list"] : array();
        if (!isset($current_settings["order"])) {
            $current_settings["order"] = "date_created";
        }
        $this->view->assign('order', $current_settings["order"]);

        if (!isset($current_settings["order_direction"])) {
            $current_settings["order_direction"] = "DESC";
        }
        $this->view->assign('order_direction', $current_settings["order_direction"]);

        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }
        $this->view->assign('page', $current_settings["page"]);

        $this->Menu_model->breadcrumbs_set_active(l("text_profile_visits", "users"));

        $this->view->render("visits", "user", "users");
    }

    /**
     * Display visits of user profile by ajax
     *
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function ajax_visits($order = "id", $order_direction = "DESC", $page = 1)
    {
        $current_settings = isset($_SESSION["users_visitors_list"]) ? $_SESSION["users_visitors_list"] : array();

        $user_id = intval($this->session->userdata("user_id"));
        $filters = array('visitor' => $user_id);

        $current_settings["order"] = $order;
        $current_settings["order_direction"] = $order_direction;
        $current_settings["page"] = $page;
        $current_settings["filters"] = $filters;
        $_SESSION["users_visitors_list"] = $current_settings;

        $this->view->output($this->_visitors_block('id_user', $order, $order_direction, $page));

        $this->view->render();
    }

    /**
     * Change view mode (list/map)
     *
     * @param string $view_mode view mode
     *
     * @return void
     */
    public function set_view_mode($view_mode)
    {
        if (in_array($view_mode, array('list', 'map'))) {
            $_SESSION['users_view_mode'] = $view_mode;
        }
        $this->view->setRedirect($_SERVER["HTTP_REFERER"]);
    }

    /**
     * Get pdf version
     *
     * @param integer $user_id user identifier
     *
     * @return void
     */
    public function pdf($user_id)
    {
        return $this->view($user_id, 'contacts', true);
    }

    /**
     * Load quick search form by ajax
     *
     * @return void
     */
    public function ajax_get_search_quick_form()
    {
        $user_type = $this->input->post('user_type', true);
        $quick_search_form_data = $this->Users_model->get_search_extend_form($user_type, 'quick_search_form_gid', false);
        if (count($quick_search_form_data) > 0) {
            $this->view->assign('form_data', $quick_search_form_data);
            $this->view->assign('form_rand', rand(100000, 999999));
            $this->view->output($this->view->fetch('helper_search_form_block', 'user', 'users'));
        }

        $this->view->render();
    }

    /**
     * Check user is authorized by ajax
     *
     * @return void
     */
    public function ajax_check_auth()
    {
        $this->view->assign('success', 1);

        $this->view->render();
    }

}
